% corPHI.m (basee sur corR.m)
% date dernieres modif. : 12/03/2018
% Macro qui permet de corriger les valeurs de J(0)
% avec plusieurs fichiers (R_gnam et dR_gnam) pour 
% pour plusieurs frequences differentes.
% 
% Vis et al., (1998) J. Biomol. NMR, 11, 265-277
%
% MODIFIEE D'APRES J-F. LEFEVRE et al., Biochemistry, 35, 2674 (1996)
% POUR SUPPRIMER L'ERREUR : CDD = 4*CDD !!!
%

grap=input(' Would you like to show graphs for each residue ? [n] : ','s');
if isempty(grap)
  grap = 'n';								% Si pas de reponse, alors "non"
elseif grap == 'y'
  figure
  hold on;
end

for k = 1:nres								% Premiere boucle pour calculer
									% les valeurs theoriques issues de la regression
  filx = wx2n;
  fily = diffr(k,:);

  xmean = sum(filx) / nfrq;
  ymean = sum(fily) / nfrq;
  diffx = filx - xmean;
  diffy = fily - ymean;
	
  varx = sum(diffx .* diffx) / nfrq;
  vary = sum(diffy .* diffy) / nfrq;
  covxy = sum(diffx .* diffy) / nfrq;
  A(1,k) = covxy / varx;						% pente
  B(1,k) = ymean - A(1,k)*xmean;					% intercept
  R(1,k) = abs(covxy / sqrt(varx*vary));				% coef. regression

  J0cor(k,1) = B(1,k) / (CDD*4);					% Valeurs ideales de la premiere correlation
  PHI(k,1) = (1/2) * (A(1,k)-(4*DELTA2*J0cor(k,1)/9));
  if PHI(k,1) < 0									% Si PHI est negatif,
    J0cor(k,1) = mean(J0(k,:));						% alors J0cor = moyenne des J0
  elseif PHI(k,1) >= 0								% sinon, J0cor est egal
    J0cor(k,1) = B(1,k) / (CDD*4);					% a l'intercept / CDD
  end

  diffrtheo(k,:) = (A(1,k) * wx2n) + B(1,k);				% Valeurs de (2*R2 - R1)  recalculees
									% pour initialiser le Monte-Carlo

  if isempty(grap)
    grap = 'n';								% Si pas de reponse, alors "non"
  elseif grap == 'y'
    errorR(1,:) = derreurs(k,:);
    errorbar(wx2n,diffr(k,:),errorR,errorR,'ro')
	hold on;
	plot(wx2n,diffrtheo(k,:),'b')
    xlabel('wx2');
    ylabel('(2*R2 - R1)');
    title(['Residu ', int2str(seq(k,1)), ', R: ',num2str(R(1,k)), ', PHI: ', num2str(PHI(k,1)) ]);
	hold off;
    pause;
  end
  
  for i = 1:nfrq							% Boucle pour eventuellement augmenter
  									% les erreurs sur certains points
    if (abs(diffr(k,i) - diffrtheo(k,i)) <= derreurs(k,i))
      derreurs_MC(k,i) = derreurs(k,i);
    else
      derreurs_MC(k,i) = abs(diffr(k,i) - diffrtheo(k,i)) + derreurs(k,i);
    end
	
  end

end

mt = 0;
while mt < 1;
  numMC = input (' Number of Monte-Carlo iterations for error calculation ? : ');
  if numMC >= 1
    mt = 1;
  end
end

for i = 1:numMC								% boucle pour le Monte-Carlo pour 
									% le calcul d'erreurs sur J(0) et PHI

% Perturbation avec une matrice aleatoire avec une distribution
% normale de moyenne 0 et de variance 1.

  aleat = randn(nres,nfrq);						% Matrice aleatoire

  if i == 1								% Le premier tour n'apporte
    aleat(:,:) = zeros(nres,nfrq);					% pas de perturbations
  end

  for k = 1:nres							% Boucle pour le 
									% calcul de la droite
    filx = wx2n;							% de regression
    fily = diffr(k,:) + (derreurs_MC(k,:) .* aleat(k,:));
    xmean = sum(filx) / nfrq;
    ymean = sum(fily) / nfrq;
    diffx = filx - xmean;
    diffy = fily - ymean;
    varx = sum(diffx .* diffx) / nfrq;
    vary = sum(diffy .* diffy) / nfrq;
    covxy = sum(diffx .* diffy) / nfrq;
    A(1,k) = covxy / varx;						% pente
    B(1,k) = ymean - A(1,k) * xmean;					% intercept

    R(i,k) = abs(covxy / sqrt(varx*vary));				% coef. regression

  end

% B = CDD.[J(0) + 3/2 J(wh)]  et  J(wh) negligeable devant J(0)

  J0perturb(i,:) = B / (CDD*4);
  PHIperturb(i,:) = (1/2)*(A-(4*DELTA2*J0perturb(i,:)/9));

end

dJ0cor = std(J0perturb);						% Expression des
dJ0cor = rot90(dJ0cor,3);						% erreurs sur J(0)
dPHI = std(PHIperturb);							% et sur PHI.
dPHI = rot90(dPHI,3);

syze = size(PHI,1);
cmpt = 0; temp = 0;
for i = 1:syze								% Recherche les negatifs
  temp = PHI(i,1);							% dans le PHI
  if temp < 0
    cmpt = cmpt + 1;
  end
end
stat0 = cmpt / syze * 100;

figure
errorbar(seq,PHI,dPHI,dPHI,'.')
hold on;
bar(seq,PHI)
title('PHI','FontSize',14)
hold off;
fprintf('\n\n')
fprintf('There are %g negative PHI values (%4.2f %%) \n',cmpt,stat0)
fprintf(' \n')

matrix(:,1) = seq;
matrix(:,2) = PHI;
matrix(:,3) = dPHI;
matrix(:,4) = J0cor;
matrix(:,5) = dJ0cor;

eval(['filr = fopen(''PHI_R.' gnout ''',''w'');']);			% Sauve la matrice des PHI
fprintf(filr,'Res       PHI         dPHI         J0cor       dJ0cor \n');

for k = 1:nres
  fprintf(filr,'%-3d  %12.5e %12.5e %12.5e %12.5e\n',matrix(k,:));
end

fclose(filr);								% Ferme le fichier 'PHI_R.out'

eval(['filrex = fopen(''Rex_R.' gnout ''',''w'');']);			% Sauve la matrice des Rex
if nfrq == 2
  fprintf(filrex,'Res    Rex_frq1     Rex_frq2\n');
elseif nfrq == 3
  fprintf(filrex,'Res    Rex_frq1     Rex_frq2     Rex_frq3\n');
elseif nfrq == 4
  fprintf(filrex,'Res    Rex_frq1     Rex_frq2     Rex_frq3     Rex_frq4\n');
elseif nfrq == 5
  fprintf(filrex,'Res    Rex_frq1     Rex_frq2     Rex_frq3     Rex_frq4     Rex_frq5\n');
elseif nfrq == 6
  fprintf(filrex,'Res    Rex_frq1     Rex_frq2     Rex_frq3     Rex_frq4     Rex_frq5     Rex_frq6\n');
end

Rex_R(:,1) = seq;
cc = nfrq + 1;								% 'cc' est le nbre total de colonne de Rex_R
Rex_R(:,2:cc) = PHI * wx2n;						% Calcul du R_exchange (Rex = PHI.wn2)

for k = 1:nres
  if nfrq == 2
    fprintf(filrex,'%-3d  %12.5e %12.5e\n',Rex_R(k,:));
  elseif nfrq == 3
    fprintf(filrex,'%-3d  %12.5e %12.5e %12.5e\n',Rex_R(k,:));
  elseif nfrq == 4
    fprintf(filrex,'%-3d  %12.5e %12.5e %12.5e %12.5e\n',Rex_R(k,:));
  elseif nfrq == 5
    fprintf(filrex,'%-3d  %12.5e %12.5e %12.5e %12.5e %12.5e\n',Rex_R(k,:));
  elseif nfrq == 6
    fprintf(filrex,'%-3d  %12.5e %12.5e %12.5e %12.5e %12.5e %12.5e\n',Rex_R(k,:));
  end
end

fclose(filrex);								% Ferme le fichier 'Rex_R.out'
fprintf(' \n')
fprintf(' Files "PHI_R.%s" and "Rex_R.%s" are created. \n',gnout,gnout);

