% dynamof.m
% date dernieres modif. : 11/03/2021
%
% Programme qui permet, dans un premier temps, de calculer 
% les valeurs de densite spectrale (J) a partir de 3 parametres 
% de relaxation  (R1, R2 et NOE) et pour N frequences.
% Puis ces valeurs de J sont ajustees par differentes variantes
% du modele de Lipari-Szabo pour obtenir les parametres du
% mouvement (Tau(c) et S2) ainsi que la contribution d'echange (Rex). 
%
% Ce programme necessite l'utilisation des macros :
% ctes_fil.m frq_pb.m c_matrix.m pjw_pb.m ceds.m corPHI.m datas.m
% metodCanet.m LSCanet.m LSCANREX.m metodLS3.m LSTauc.m calc_tauc.m
% LSREX.m LS2.m LSE3.m LSE4.m introdynamof.m
%
% Remarque : Le calcul des J est derive du programme MATLAB 
% de J-F. Lefevre et ses collaborateurs : init.m
%
% Commentaires : p.barthe@cbs.cnrs.fr ou c.roumestand@cbs.cnrs.fr
%

introdynamof                                                                % Macro qui dessine "dynamof"

fprintf(' last modif. : 2021/03/11 \n')
fprintf('\n')

clear
                                                                            % Declaration des variables globales
global tauc lwx2;

fprintf('\n')
fprintf(' You must prepare your files, such as: R1_test.txt, R2_test.txt ans NOE_test.txt \n')
fprintf(' where "test" is the generic name of your study. \n')
fprintf('\n')

% Declaration des variables
gnout = []; plotadr = []; perso = []; DELTA  = []; nfrq = [];
fmy = []; fr2 = []; gnamr2 = []; frnoe = []; gnamnoe = [];
nuh = []; gnam = []; cory = []; metod = []; tauc = []; 
rexy = []; rtau = []; graf = []; inmc = []; numMC = [];
lamwx2 = []; lwx2 = [];
cory = 'n';

gnout = input(' What suffix for output files ? : ','s');

plotadr = input(' File name of output plot directory ? [./fit_plots] : ','s');
if isempty(plotadr)
  plotadr = './fit_plots';                                                  % Declaration du repertoire ou seront stockes les figures
end

perso = input(' Would you like to personalise some variables ? [y/n] : ','s');
if isempty(perso)
  perso = 'n';
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  CHARGEMENT DES CONSTANTES

input(' Type of X nucleus: N or C  ? : ','s');
if ans == 'N'
  nuc = 1;
  gamx = -2.711e7;				% Rapport gyromagn. du 15N (rad.s-1.T-1)
  rhx = 1.02e-10;				% Distance N-H
  if perso == 'y'
    DELTA = input(' CSA value for 15N nucleus (ppm) ? [-170 ppm] : ') / 1e6;
    % CSA convertie en ppm
  end
  if isempty(DELTA)
    DELTA = -170e-6;			% Si pas de reponse, alors "-170 ppm" pour le 15N
  end
  fprintf('\n')
  fprintf('  CSA value for 15N = %g ppm\n', DELTA * 1e6)
elseif ans == 'C'
  nuc = 2;
  gamx = 6.726e7;				% Rapport gyromagn. du 13C (rad.s-1.T-1)
  rhx = 1.07e-10;				% Distance C-H
  if perso == 'y'
    DELTA = input(' CSA value for 13C nucleus (ppm) ? [25 ppm] : ') / 1e6;
    % CSA convertie en ppm
  end
  if isempty(DELTA)
    DELTA = 25e-6;				% Si pas de reponse, alors "25 ppm" pour le 13C
  end
  fprintf('\n')
  fprintf('  CSA value for 13C = %g ppm\n', DELTA * 1e6)
else
  fprintf('\n')
  error('Error. You must answer ''N'' or ''C'' ');
end

ctes_fil                        % UTILISATION de 'ctes_fil.m'


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  PETIT QUESTIONNAIRE

fprintf('\n')
nfrq = input(' Number of different frequencies ? : ');
if (isempty(nfrq) | nfrq == 0)
  fprintf('\n')
  error('Error, ''nfrq'' paremeter must be up to ''0'' ');
elseif nfrq > 6
  fprintf('\n')
  error('Error, ''nfrq'' paremeter exceded ''6'' ');
end

%%%  xj0 = input(' Do you want to use the J(0) values for this calcul ? [y/n] : ','s');		% OLD
xj0 = 'y';

if nfrq == 1                                                                % Si une seule frequence...
  xj0 = 'y';                                                                % alors toujours utilisation des J(0)
  fprintf('\n Give following datas... \n');
else                                                                        % Sinon, plusieurs frequences
  fprintf('\n')
  fmy = input(' Do you want to use the ''Fast-mapping'' approach ? [n] : ','s');
  if fmy == 'y'
    fr2 = input(' Proton frequency for R2 calculation (MHz) ? : ') * 1e6;	% Frequence des R2 convertie en Hz
    gnamr2 = input ('   Generic name ? : ','s');                            % Recupere le 'gnam' des R2
    frnoe = input(' Proton frequency for NOE calculation (MHz) ? : ') * 1e6;	% Frequence des NOE convertie en Hz
    gnamnoe = input ('   Generic name ? : ','s');                           % Recupere le 'gnam' des NOE
  end
  fprintf('\n\n\n Give files in frequency growing order (Ex: 400, 500 then 600) \n');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  CHARGEMENT DES R ET CALCUL DES J

for i = 1:nfrq

  fprintf('Frequency %g : \n',i);
  nuh = input('   Proton frequency (MHz) ? : ') * 1e6;                      % Frequence convertie en Hz
  nuhn(1,i) = nuh;

  frq_pb                                                                    % UTILISATION de 'frq.m' pour le
                                                                            % calcul des frequences de Larmor
  wx2n(1,i) = wx2;                                                          % du noyau X
  nuxf(:,i) = nux;                                                          % Valeur de nu(x)

  if fmy == 'y'
    mfrnoe = frnoe * 0.87;
  else
    mnuhf(:,i) = nuh * 0.87;                                                % Valeur de nu(h) modifiee a 0.87
  end

  gnam = input ('   Generic name ? : ','s');
  eval(['gnam_' num2str(i) ' =  gnam;']);                                   % Recupere le 'gnam'
  fprintf('\n')

%%% 1 : Mettre un test pour verifier que les fichiers R1,R2,NOE aient le meme nbre de lignes

  eval(['load R1_' gnam '.txt ;']);                                          % Charge le fichier R1_ xxx .txt
  eval(['r1data = R1_' gnam ' ;']);

  if fmy == 'y'                                                             % Si "Fast-mapping" (1 R2, 1 NOE and X R1 )
    eval(['load R2_' gnamr2 '.txt ;']);                                     % Charge le seul fichier R2_ xxx .txt
    eval(['r2data = R2_' gnamr2 ' ;']);
    eval(['load NOE_' gnamnoe '.txt ;']);                                   % Charge le seul fichier NOE_ xxx .txt
    eval(['noedata = NOE_' gnamnoe ' ;']);
  else                                                                      % Si calcul classique (1 R1, 1 R2 and 1 NOE for each frequency)
    eval(['load R2_' gnam '.txt ;']);                                       % Charge le fichier R2_ xxx .txt
    eval(['r2data = R2_' gnam ' ;']);
    eval(['load NOE_' gnam '.txt ;']);                                          % Charge le fichier NOE_ xxx .txt
    eval(['noedata = NOE_' gnam ' ;']);
  end
                                                                            % Build R_matrix (R1, R2 and NOE)
  rdata(:,1) = r1data(:,2);                                                 % R1 values
  rdata(:,2) = r2data(:,2);                                                 % R2 values
  rdata(:,3) = (noedata(:,2)-1).*(rdata(:,1)*gamx/gamh);                    % NOE values converted in SIGMA values
  
  [nrr ncr] = size(rdata);
 
  seq = r1data(:,1);					% Charge une sequence

  CDD = ((mu0/(4*pi))^2)*(h_bar2*gamx2*gamh2)/(4*rhx6);	% CDD: Constante d'interaction dipolaire

  CCSA = (DELTA2 * wx2)/3;				% CCSA: Constante d'anisotropie de deplacement chimique

  c_matrix						% UTILISATION de 'c_matrix.m'
  							% (modifiee pour corriger le 
							% calcul du CDD.)

  % invert the matrix and compute J values
  matc = matc3;
  matr = inv(matc);
  for wi = 1:nrr;
	vr = rdata(wi,:);
	vj = matr*vr';
	jdata(wi,:) = vj(:)';
  end


  if fmy == 'y'
    eval(['save J_' gnam '_FM.' gnout ' jdata -ascii;']);	% Sauve les valeurs de J
    [nrj ncj] = size(jdata);
    fprintf(' Files created: J_%s_FM.%s',gnam,gnout);
  else
    eval(['save J_' gnam '.' gnout ' jdata -ascii;']);		% Sauve les valeurs de J
    [nrj ncj] = size(jdata);
    fprintf(' Files created: J_%s.%s',gnam,gnout);
  end
  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  CHARGEMENT DES dR ET CALCUL DES dJ

  dr1 = r1data(:,3);									% dR1 values
  dr2 = r2data(:,3);									% dR2 values
  dsig = (abs((noedata(:,3)./noedata(:,2)))+(dr1(:,1)./r1data(:,2))).*rdata(:,3);	% dNOE values converted in dSIGMA values
  
  ceds							% UTILSATION de 'ceds.m' modifiee
							% pour calculer les erreurs														
  if fmy == 'y'
    eval(['save dJ_' gnam '_FM.' gnout ' djdata -ascii;']);	% Sauve les valeurs de dJ
    fprintf(' and dJ_%s_FM.%s \n',gnam,gnout);
  else
    eval(['save dJ_' gnam '.' gnout ' djdata -ascii;']);	% Sauve les valeurs de dJ
    fprintf(' and dJ_%s.%s \n',gnam,gnout);
  end

  eval(['jdata_' gnam ' = jdata;']);			% Recupere la matrice "jdata" (pour corPHI.m)
  eval(['djdata_' gnam ' = djdata;']);			% Recupere la matrice "djdata" (pour corPHI.m)

  % plot for check
  pjw_pb						% UTILISATION de 'pjw.m' pour tracer les graphes des J

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  SAUVE LES VALEURS DE J ET dJ

  if fmy == 'y'

    if nuh == fr2
      J0(:,1)  =  jdata(:,1);				% Recupere les J(0) calcule avec les R2 
      dJ0(:,1) = djdata(:,1);				% ainsi que les dJ(0)
    end
    Jwx(:,i)  =  jdata(:,2);				% Recupere les J(wx)
    dJwx(:,i) = djdata(:,2);				% et les dJ(wx) pour chaque frequence
    if nuh == frnoe
      Jwh(:,1)  =  jdata(:,3);				% Recupere les <J(wh)> calcule avec les NOE
      dJwh(:,1) = djdata(:,3);				% ainsi que les d<J(wh)>
    end

  else

    J0(:,i)  = jdata(:,1);				% Recupere les J(0)
    Jwx(:,i) = jdata(:,2);				% Recupere les J(wx)
    Jwh(:,i) = jdata(:,3);				% Recupere les <J(wh)>

    dJ0(:,i)  = djdata(:,1);				% Recupere les dJ(0)
    dJwx(:,i) = djdata(:,2);				% Recupere les dJ(wx)
    dJwh(:,i) = djdata(:,3);				% Recupere les d<J(wh)>

  end

  lambda(1,i) = (3/2) / (3*CDD + CCSA);			% Calcul de lambda pour chaque frequence
  lamwx2(1,i) = lambda(1,i) * wx2n(1,i);		% Calcul de lambda.wx2

  if (nfrq > 1 & fmy ~= 'y')				% Necessaire pour eventuellement utiliser corPHI.m
    diffr(:,i) = (2*rdata(:,2)) - rdata(:,1);		% difference des R (2*R2 - R1)
	derreurs(:,i) = (dr1 + 2*dr2);			% somme des erreurs
  end

end

nres = size(seq,1);					% Nombre de residus


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  CORRECTION DES J(0) PAR COR_PHI ?

if (nfrq > 1 & fmy ~= 'y')				% Si plus d'1 freq et pas de "Fast-mapping"
  fprintf('\n')
  cory = input(' Do you want to correct the ''J(0)'' values ? [y/n] : ','s');

  if cory == 'y'					% Si correction des J(0)
  
    corPHI						% UTILISATION de 'corPHI.m'

%%%    fprintf('\n')
%%%    rep=input(' Do you want to replace experimental J(0) values by corrected J(0) values ? [n] : ','s');

%%%    if rep == 'y'

        fprintf('\n')
        fprintf('  ######################################\n')
        fprintf('  # J(0) values will be corrected now. #\n')
        fprintf('  ######################################\n')
        fprintf('\n')

      fprintf(' Press any key to continue...\n')
      pause;

      for i = 1:nfrq
        J0(:,i) = J0cor;					% Remplace les valeurs de J(0)exp par les J(0)cor
	
        eval(['gnam = gnam_' num2str(i) ' ;']);			% Recupere le gnam de chaque fichier
	
        eval(['J_cor = jdata_' gnam ';']);			% Recupere les valeurs de J experimentales
        J_cor(:,1) = J0cor;					% Remplace les J(0)exp par J(0)cor
        eval(['save J_' gnam 'corR.' gnout ' J_cor -ascii;']);	% puis sauve le fichier 'J_...corR'
	
        eval(['dJ_cor = djdata_' gnam ';']);			% Recupere les valeurs de dJ experimentales
        dJ_cor(:,1) = dJ0cor;					% Remplace les dJ(0)exp par dJ(0)cor
        eval(['save dJ_' gnam 'corR.' gnout ' dJ_cor -ascii;']);% puis sauve le fichier 'dJ_...corR'	
        fprintf('\n')
        fprintf(' Files "J_%scorR.%s" and "dJ_%scorR.%s" are created.\n',gnam,gnout,gnam,gnout);
      end
%%%    end

  elseif (cory ~= 'y' & cory ~= 'n')
    fprintf('\n')
	error('Error. You must answer ''y'' or ''n'' ');
  end
end

fprintf('\n')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  MISE EN PLACE DE VARIABLES DIVERSES

NuX(1,1) = 0;						% Determination des Omega
NuX(1,2:(nfrq+1)) = abs(nuxf);

if fmy == 'y'						% Si "Fast-Mapping"
  NuX(1,(nfrq+2)) = mfrnoe;				% La frequence des NOE (Hz) modifiee a 0.87
else							% Sinon
  NuX(1,(nfrq+2):(2*nfrq+1)) = mnuhf;			% Les valeurs nu(h) modifiees a 0.87
end

OmegaX = NuX*(2*pi);

% Pour faire un fit de 100 points
if fmy == 'y'						% Si "Fast-Mapping"
  fNuX = [NuX(1,1):NuX(1,(nfrq+2))/100:NuX(1,(nfrq+2))];
  fX = [OmegaX(1,1):OmegaX(1,(nfrq+2))/100:OmegaX(1,(nfrq+2))];
else
  fNuX = [NuX(1,1):NuX(1,(2*nfrq+1))/100:NuX(1,(2*nfrq+1))];
  fX = [OmegaX(1,1):OmegaX(1,(2*nfrq+1))/100:OmegaX(1,(2*nfrq+1))];
end

X = OmegaX;
for k = 1:nres
  if fmy == 'y'						% Si "Fast-Mapping", determination des valeurs (Jomega)
    Jomega(k,1) = mean(J0(k,1));			% J(0) a la frequence R2
    Jomega(k,2:(nfrq+1)) = Jwx(k,:);			% Jwx pour toutes les frequences
	Jomega(k,(nfrq+2)) = Jwh(k,1);			% Jwh a la frequence du NOE
  else							% sinon
    Jomega(k,1) = mean(J0(k,:));			% J(0) pour toutes les frequences
    Jomega(k,2:(nfrq+1)) = Jwx(k,:);			% Jwx pour toutes les frequences
    Jomega(k,(nfrq+2):(2*nfrq+1)) = Jwh(k,:);		% Jwh pour toutes les frequences
  end

  if fmy == 'y'
	dJomega(k,1) = mean(dJ0(k,1));			% Determination des erreurs (dJomega)
    dJomega(k,2:(nfrq+1)) = dJwx(k,:);
	dJomega(k,(nfrq+2)) = dJwh(k,1);
  else
	dJomega(k,1) = mean(dJ0(k,:));			% Determination des erreurs (dJomega)
    dJomega(k,2:(nfrq+1)) = dJwx(k,:);
	dJomega(k,(nfrq+2):(2*nfrq+1)) = dJwh(k,:);
  end
end

datas							% UTILISATION de 'datas.m'
							% qui demande les valeurs de S2 et Tau
														
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% CHOIX DE L'APPROCHE : LOCALE OU GLOBALE ?

play = 'Q';						% Variable qui permet de Quitter le programme ulterieurement

fprintf('\n')
metod = input(' Which method you want (GLOBAL or LOCAL) to analyse your datas ? [G/L] : ','s');

if (metod ~= 'G' & metod ~= 'L')
  fprintf('\n')
  error('Error, You must answer ''G'' or ''L'' ');

elseif metod == 'L'					% Si approche LOCALE (Selon Canet et al., JACS 2001)
  
  fprintf('\n')
  tauc = input(' Give an approximative Tau(c) value (ns) (to initialize fits): ');
  tauc = tauc * 1e-9;

  if cory ~= 'y'					% Si on n'a pas fait COR_PHI
    fprintf('\n')
    rexy = input(' Do you want to fit the exchange parameter ''Rex'' ? [n] : ','s');
  end

  if rexy == 'y'
    lwx2 = mean(lamwx2);		%%%%%%%%%%%%%%% ????? 08/12/04 ????? %%%%%%%%%%%%%%%
  end

  metodCanet						% UTILISATION de 'metodCanet.m'	et 'LSCanet.m'

elseif metod == 'G'					% Si approche GLOBALE (Lipari-Szabo classiqe)

  play = 'C';						% Variable qui permet de Poursuivre le programme ulterieurement

  fprintf('\n')
  rtau = input(' Do you want to calculate Tau(c) ? [n] : ','s');

  if rtau == 'y'					% Si oui, alors on calcule

    fprintf('\n')
    tauc = input(' Give an approximative Tau(c) value (ns) (to initialize fits): ');
    tauc = tauc * 1e-9;

    metodLS3						% UTILISATION de 'metodLS3.m' et 'LSTauc.m'

    calc_tauc						% UTILISATION de 'calc_tauc.m'

  end

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DEPART DU PROGRAMME D'AJUSTEMENT DES J

if play == 'C'

  fprintf('\n\n ################################################ \n\n')
  fprintf('     Spectral densities will be fitted now...  ')
  
  fprintf('\n')
  tauc = input(' Tau(c) value (ns) ? : ');
  tauc = tauc * 1e-09;

  if cory ~= 'y'					% Si on n'a pas fait COR_PHI
    fprintf('\n')
    rexy = input(' Do you want to fit the exchange parameter ''Rex'' ? [n] : ','s');
  end

  if rexy == 'y'					% Si calcul du Rex
    lwx2 = mean(lamwx2);		%%%%%%%%%%%%%%% ????? 08/12/04 ????? %%%%%%%%%%%%%%%

    eval(['fil1 = fopen(''lsrex.' num2str(gnout) ''',''w'');']);	% Ouvre le fichier 'lsrex.out'
    if nfrq == 1
      fprintf(fil1,'Res    S2      Tf_ps     Chi2        R       Tc        J(0)        Phi      Rex\n');
    elseif nfrq == 2
      fprintf(fil1,'Res    S2      Tf_ps     Chi2        R       Tc        J(0)        Phi     Rex1   Rex2\n');
    elseif nfrq == 3
      fprintf(fil1,'Res    S2      Tf_ps     Chi2        R       Tc        J(0)        Phi     Rex1   Rex2   Rex3\n');
    elseif nfrq == 4
      fprintf(fil1,'Res    S2      Tf_ps     Chi2        R       Tc        J(0)        Phi     Rex1   Rex2   Rex3   Rex4\n');
    elseif nfrq == 5
      fprintf(fil1,'Res    S2      Tf_ps     Chi2        R       Tc        J(0)        Phi     Rex1   Rex2   Rex3   Rex4   Rex5\n');
    elseif nfrq == 6
      fprintf(fil1,'Res    S2      Tf_ps     Chi2        R       Tc        J(0)        Phi     Rex1   Rex2   Rex3   Rex4   Rex5   Rex6\n');
    end
  end

  eval(['fil2 = fopen(''ls2.' num2str(gnout) ''',''w'');']);		% Ouvre le fichier 'ls2.out'
  eval(['fil3 = fopen(''lse3.' num2str(gnout) ''',''w'');']);		% Ouvre le fichier 'lse3.out'
  fprintf(fil2,'Res    S2      Tf_ps     Chi2        R       Tc\n');
  fprintf(fil3,'Res   S2s    S2f   S2s*S2f   Ts_ps     Chi2        R       Tc\n');

  if nfrq > 1						% Si plusieurs frequences...
    eval(['fil4 = fopen(''lse4.' num2str(gnout) ''',''w'');']);	% Ouvre le fichier 'lse4.out'
    fprintf(fil4,'Res   S2s    S2f   S2s*S2f   Ts_ps     Tf_ps     Chi2        R       Tc\n');
  end

  fprintf('\n')
  graf = input(' Would you like to show graphs for each residue ? [n] : ','s');
  
  if graf == 'y'
    figure						% Creation d'une nouvelle figure
    whitebg([0 0 0]);					% Background de la figure en NOIR 
  end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% CALCUL DES PREMIERES VALEURS

  for k = 1:nres					% Premiere boucle pour calculer
							% les valeurs theoriques avant le MC
    fprintf('Residu: %g \n',seq(k,1));

    Y = Jomega(k,:);
    dY = dJomega(k,:);

    option=optimset('fminsearch');
%    option=optimset(option,'Display','Notify','MaxIter',40000,'MaxFunEvals',50000);

    if rexy == 'y'					% Si calcul du Rex
							% Initialisation du fit LSREX
      phi = 10.^(-15);					% Valeur de Phi # 10^(-15) s.rad-2
      params = [s2 taup phi];
      fitLSREX = fminsearch('LSREX',params,option,X,Y,dY,tauc,lwx2);	% UTILISATION de 'LSREX.m' pour le fit
      calcYlsrex(k,:) = (2/5)*((fitLSREX(1)*tauc./(1+(X*tauc).^2))+((1-fitLSREX(1))*fitLSREX(2)./(1+(X*fitLSREX(2)).^2)))+(lwx2*abs(fitLSREX(3))*(exp(-(X).^6)));
      calcYlsrex_Rex(k,:) = (2/5)*((fitLSREX(1)*tauc./(1+(X*tauc).^2))+((1-fitLSREX(1))*fitLSREX(2)./(1+(X*fitLSREX(2)).^2)));		% ATTENTION, J(0) = J(0)obs - lambda.Rex
      params_LSREX(k,1) = seq(k,1);
      params_LSREX(k,2) = fitLSREX(1);					% Valeur du S2
      params_LSREX(k,3) = (tauc*fitLSREX(2))/(tauc-fitLSREX(2))*1e12;	% Conversion taup > tauf
      error2 = (Y - calcYlsrex(k,:)).^2;
      params_LSREX(k,4) = sum(error2./(calcYlsrex(k,:).^2));		% Calcul du Chi2
      R2 = corrcoef(Y,calcYlsrex(k,:));
      params_LSREX(k,5) = R2(1,2);					% Calcul du R
      params_LSREX(k,6) = tauc;						% Valeur du Tau(c)
      params_LSREX(k,7) = calcYlsrex_Rex(k,1);				% Valeur de J(0) = J(0)obs - lambda.Rex
      params_LSREX(k,8) = abs(fitLSREX(3));				% Valeur du Phi (s.rad-2)
      if nfrq == 1
        params_LSREX(k,9) = wx2n(1,:) * abs(fitLSREX(3));		% Valeur du Rex = wx2 . phi
        fprintf(fil1,'%-3d  %5.2f  %8.2f  %7.5e  %5.3f  %5.2e  %7.5e  %5.2e  %5.3e\n',params_LSREX(k,:));
      elseif nfrq == 2
        params_LSREX(k,9:10) = wx2n(1,:)  * abs(fitLSREX(3));
        fprintf(fil1,'%-3d  %5.2f  %8.2f  %7.5e  %5.3f  %5.2e  %7.5e  %5.2e  %5.3e  %5.3e\n',params_LSREX(k,:));
      elseif nfrq == 3
        params_LSREX(k,9:11) = wx2n(1,:)  * abs(fitLSREX(3));
        fprintf(fil1,'%-3d  %5.2f  %8.2f  %7.5e  %5.3f  %5.2e  %7.5e  %5.2e  %5.3e  %5.3e  %5.3e\n',params_LSREX(k,:));
      elseif nfrq == 4
        params_LSREX(k,9:12) = wx2n(1,:)  * abs(fitLSREX(3));
        fprintf(fil1,'%-3d  %5.2f  %8.2f  %7.5e  %5.3f  %5.2e  %7.5e  %5.2e  %5.3e  %5.3e  %5.3e  %5.3e\n',params_LSREX(k,:));
      elseif nfrq == 5
        params_LSREX(k,9:13) = wx2n(1,:)  * abs(fitLSREX(3));
        fprintf(fil1,'%-3d  %5.2f  %8.2f  %7.5e  %5.3f  %5.2e  %7.5e  %5.2e  %5.3e  %5.3e  %5.3e  %5.3e  %5.3e\n',params_LSREX(k,:));
      elseif nfrq == 6
        params_LSREX(k,9:14) = wx2n(1,:)  * abs(fitLSREX(3));
        fprintf(fil1,'%-3d  %5.2f  %8.2f  %7.5e  %5.3f  %5.2e  %7.5e  %5.2e  %5.3e  %5.3e  %5.3e  %5.3e  %5.3e  %5.3e\n',params_LSREX(k,:));
      end

      initprm_LSREX(k,1) = fitLSREX(1);					% Valeur du S2
      initprm_LSREX(k,2) = fitLSREX(2);					% Valeur du taup (en s)
      initprm_LSREX(k,3) = abs(fitLSREX(3));				% Valeur du Phi (s.rad-2)

    end


									% Initialisation du fit LS2
    params = [s2 taup];
    fitLS2 = fminsearch('LS2',params,option,X,Y,dY,tauc);		% UTILISATION de 'LS2.m' pour le fit
    calcYls2(k,:) = (2/5)*((fitLS2(1)*tauc./(1+(X*tauc).^2))+((1-fitLS2(1))*fitLS2(2)./(1+(X*fitLS2(2)).^2)));
    params_LS2(k,1) = seq(k,1);
    params_LS2(k,2) = fitLS2(1);					% Valeur du S2
    params_LS2(k,3) = (tauc*fitLS2(2))/(tauc-fitLS2(2))*1e12;		% Conversion taup > tauf
    error2 = (Y - calcYls2(k,:)).^2;
    params_LS2(k,4) = sum(error2./(calcYls2(k,:).^2));			% Calcul du Chi2
    R2 = corrcoef(Y,calcYls2(k,:));
    params_LS2(k,5) = R2(1,2);						% Calcul du R
    params_LS2(k,6) = tauc;
%%%    params_LS2(k,7) = calcYls2(k,1);					% Valeur de J(0) = J(0)obs - lambda.Rex
%%%    fprintf(fil2,'%-3d  %5.2f  %8.2f  %7.5e  %5.3f  %5.2e  %7.5e\n',params_LS2(k,:));
    fprintf(fil2,'%-3d  %5.2f  %8.2f  %7.5e  %5.3f  %5.2e\n',params_LS2(k,:));

    initprm_LS2(k,1) = fitLS2(1);					% Valeur du S2
    initprm_LS2(k,2) = fitLS2(2);					% Valeur du taup (en s)


									% Initialisation du fit LSE3
    params = [s2s s2f tausp];
    fitLSE3 = fminsearch('LSE3',params,option,X,Y,dY,tauc);		% UTILISATION de 'LSE3.m' pour le fit
    calcYlse3(k,:) = (2/5)*((fitLSE3(1)*fitLSE3(2)*tauc./(1+(X*tauc).^2))+(fitLSE3(2)*(1-fitLSE3(1))*fitLSE3(3)./(1+(X*fitLSE3(3)).^2)));
    params_LSE3(k,1) = seq(k,1);
    params_LSE3(k,2) = fitLSE3(1);					% Valeur du S2s
    params_LSE3(k,3) = fitLSE3(2);					% Valeur du S2f
    params_LSE3(k,4) = fitLSE3(1) * fitLSE3(2);				% Calcul de S2s*S2f
    params_LSE3(k,5) = (tauc*fitLSE3(3))/(tauc-fitLSE3(3))*1e12;	% Conversion tausp > taus
    error2 = (Y - calcYlse3(k,:)).^2;
    params_LSE3(k,6) = sum(error2./(calcYlse3(k,:).^2));		% Calcul du Chi2
    R2 = corrcoef(Y,calcYlse3(k,:));
    params_LSE3(k,7) = R2(1,2);						% Calcul du R
    params_LSE3(k,8) = tauc;
%%%    params_LSE3(k,9) = calcYlse3(k,1);				% Valeur de J(0) = J(0)obs - lambda.Rex
%%%    fprintf(fil3,'%-3d  %5.2f  %5.2f  %5.2f  %8.2f  %7.5e  %5.3f  %5.2e  %7.5e\n',params_LSE3(k,:));
    fprintf(fil3,'%-3d  %5.2f  %5.2f  %5.2f  %8.2f  %7.5e  %5.3f  %5.2e\n',params_LSE3(k,:));

    initprm_LSE3(k,1) = fitLSE3(1);					% Valeur du S2s
    initprm_LSE3(k,2) = fitLSE3(2);					% Valeur du S2f
    initprm_LSE3(k,3) = fitLSE3(3);					% Valeur du tausp (en s)


    if nfrq > 1								% Initialisation du fit LSE4
      params = [s2s s2f tausp taufp];
      fitLSE4 = fminsearch('LSE4',params,option,X,Y,dY,tauc);		% UTILISATION de  'LSE4.m' pour le fit
      calcYlse4(k,:) = (2/5)*((fitLSE4(1)*fitLSE4(2)*tauc./(1+(X*tauc).^2))+(fitLSE4(2)*(1-fitLSE4(1))*fitLSE4(3)./(1+(X*fitLSE4(3)).^2))+((1-fitLSE4(2))*fitLSE4(4)./(1+(X*fitLSE4(4)).^2)));
      params_LSE4(k,1) = seq(k,1);
      params_LSE4(k,2) = fitLSE4(1);					% Valeur du S2s
      params_LSE4(k,3) = fitLSE4(2);					% Valeur du S2f
      params_LSE4(k,4) = fitLSE4(1) * fitLSE4(2);			% Calcul de S2s*S2f
      params_LSE4(k,5) = (tauc*fitLSE4(3))/(tauc-fitLSE4(3))*1e12;	% Conversion tausp > taus
      params_LSE4(k,6) = (tauc*fitLSE4(4))/(tauc-fitLSE4(4))*1e12;	% Conversion taufp > tauf
      error2 = (Y - calcYlse4(k,:)).^2;
      params_LSE4(k,7) = sum(error2./(calcYlse4(k,:).^2));		% Calcul du Chi2
      R2 = corrcoef(Y,calcYlse4(k,:));
      params_LSE4(k,8) = R2(1,2);					% Calcul du R
      params_LSE4(k,9) = tauc;
%%%      params_LSE4(k,10) = calcYlse4(k,1);				% Valeur de J(0) = J(0)obs - lambda.Rex
%%%      fprintf(fil4,'%-3d  %5.2f  %5.2f  %5.2f  %8.2f  %8.2f  %7.5e  %5.3f  %5.2e  %7.5e\n',params_LSE4(k,:));
      fprintf(fil4,'%-3d  %5.2f  %5.2f  %5.2f  %8.2f  %8.2f  %7.5e  %5.3f  %5.2e\n',params_LSE4(k,:));

      initprm_LSE4(k,1) = fitLSE4(1);					% Valeur du S2s
      initprm_LSE4(k,2) = fitLSE4(2);					% Valeur du S2f
      initprm_LSE4(k,3) = fitLSE4(3);					% Valeur du tausp (en s)
      initprm_LSE4(k,4) = fitLSE4(4);					% Valeur du taufp (en s)
    end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% FAIRE LES GRAPHES

    if isempty(graf)

      graf = 'n';

    elseif graf == 'y'							% Si 'y', alors on trace ...

    subplot(1,2,1)							% Graphe de gauche
      errorJ(1,:) = dJomega(k,:);					% Calcul de l'erreur
      errorbar((X/(2*pi))/1e6,Jomega(k,:),errorJ,errorJ,'wo')		% ...les valeurs experimentales (avec erreurs)
      hold on;
      if rexy == 'y'							% Si calcul du Rex
        calcLSREX_REX = (2/5)*((fitLSREX(1)*tauc./(1+(fX*tauc).^2))+((1-fitLSREX(1))*fitLSREX(2)./(1+(fX*fitLSREX(2)).^2)));	% ATTENTION, J(0) = J(0)obs - lambda.Rex
        plot(fNuX/1e6,calcLSREX_REX,'y')				% ...le fit LSREX
      end
      calcLS2 = (2/5)*((fitLS2(1)*tauc./(1+(fX*tauc).^2))+((1-fitLS2(1))*fitLS2(2)./(1+(fX*fitLS2(2)).^2)));
      plot(fNuX/1e6,calcLS2,'r')					% ...le fit LS2
      calcLSE3 = (2/5)*((fitLSE3(1)*fitLSE3(2)*tauc./(1+(fX*tauc).^2))+(fitLSE3(2)*(1-fitLSE3(1))*fitLSE3(3)./(1+(fX*fitLSE3(3)).^2)));
      plot(fNuX/1e6,calcLSE3,'g')					% ...le fit LSE3
      if nfrq > 1
        calcLSE4 = (2/5)*((fitLSE4(1)*fitLSE4(2)*tauc./(1+(fX*tauc).^2))+(fitLSE4(2)*(1-fitLSE4(1))*fitLSE4(3)./(1+(fX*fitLSE4(3)).^2))+((1-fitLSE4(2))*fitLSE4(4)./(1+(fX*fitLSE4(4)).^2)));
        plot(fNuX/1e6,calcLSE4,'b')					% ...le fit LSE4
      end
      xlabel('Frequency (MHz)')
      ylabel('J(\omega) (s.rad-1)')
      hold off;

    subplot(1,2,2)							% Graphe de droite (en log)
      errorJ_log(1,:) = log(Jomega(k,:)) - (log(Jomega(k,:)-dJomega(k,:)));	% Calcul de l'erreur
      errorbar((X/(2*pi))/1e6,log(Jomega(k,:)),errorJ_log,errorJ_log,'wo')	% ...les valeurs experimentales (avec erreurs)
      hold on;
      if rexy == 'y'							% Si calcul du Rex
        plot(fNuX/1e6,log(calcLSREX_REX),'y')				% ...le fit LSREX
      end
      plot(fNuX/1e6,log(calcLS2),'r')					% ...le fit LS2
      plot(fNuX/1e6,log(calcLSE3),'g')					% ...le fit LSE3
      if nfrq > 1
        plot(fNuX/1e6,log(calcLSE4),'b')				% ...le fit LSE4
      end

      xlabel('Frequency (MHz)')
      ylabel('log(J(\omega)) (s.rad-1)')
      title(['Residu ', int2str(seq(k,1))])

      if rexy == 'y'							% Si calcul du Rex...
        if nfrq == 1							% Si une seule frequence...
          legend('data','LSREX','LS2','LSE3')
          HL = legend('data','LSREX','LS2','LSE3');
        else								% Si plusieurs frequences...
          legend('data','LSREX','LS2','LSE3','LSE4')
          HL = legend('data','LSREX','LS2','LSE3','LSE4');
        end
      else								% Si PAS calcul du Rex...
        if nfrq == 1							% Si une seule frequence...
          legend('data','LS2','LSE3')
          HL = legend('data','LS2','LSE3');
        else								% Si plusieurs frequences...
          legend('data','LS2','LSE3','LSE4')
          HL = legend('data','LS2','LSE3','LSE4');
        end
      end

      hold off;

      fprintf(' Press any key to continue...\n')
      pause;

      set(HL,'TextColor','black');						% Change la couleur de la legende avant de 
      eval(['print -depsc ', plotadr,'/fit',metod ,'_', int2str(seq(k,1))]);	% Sauver les figures dans "fit_plots"

    end
  end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if rexy == 'y'							% Si calcul du Rex...
    fclose(fil1);							% Ferme le fichier "lsrex.out"
    eval(['save Jtheo_lsrex.' num2str(gnout) ' calcYlsrex_Rex -ascii;']);	% ATTENTION, J(0) = J(0)obs - lambda.Rex
    fprintf(' \n')
    fprintf(' Files "lsrex.%s" and "Jtheo_lsrex.%s" are created. \n',gnout,gnout);
  end

  fclose(fil2);								% Ferme le fichier "ls2.out"
  fclose(fil3);								% Ferme le fichier "lse3.out"

  if nfrq > 1
    fclose(fil4);							% Ferme le fichier "lse4.out"
  end

  eval(['save Jtheo_ls2.' num2str(gnout) ' calcYls2 -ascii;']);
  eval(['save Jtheo_lse3.' num2str(gnout) ' calcYlse3 -ascii;']);
  fprintf(' \n')
  fprintf(' Files "ls2.%s" and "Jtheo_ls2.%s" are created. \n',gnout,gnout);
  fprintf(' \n')
  fprintf(' Files "lse3.%s" and "Jtheo_lse3.%s" are created. \n',gnout,gnout);

  if nfrq > 1
    eval(['save Jtheo_lse4.' num2str(gnout) ' calcYlse4 -ascii;']);
    fprintf(' \n')
    fprintf(' Files "lse4.%s" and "Jtheo_lse4.%s" are created. \n',gnout,gnout);

  end

  if graf == 'y'
    fprintf('\n')
    fprintf(' Graphs are saved in "%s" directory. \n',plotadr);
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% CALCUL PAR MONTE-CARLO

  fprintf('\n')
  mt = 0;
  while mt < 1;
    inmc = input (' Do you want to do a Monte-Carlo calculation ? [y] or [n] : ','s');
    if inmc == 'n'
      mt = 1;
    elseif inmc == 'y'
      mt = 1;
    end
  end

  if inmc == 'y'							% Depart du Monte-Carlo

    numMC = input (' Number of MC iterations ?  ');

    if rexy == 'y'							% Si calcul du Rex...
      eval(['fil1MC = fopen(''lsrex_MC.' num2str(gnout) ''',''w'');']);	% Ouvre 'lsrex_MC.out'
      if nfrq == 1
        fprintf(fil1MC,'Res    S2   dS2     Tf_ps   dTf_ps     Chi2       dChi2        R    dR      Tc      numMC    J(0)       dJ(0)        Phi     dPhi      Rex   dRex\n');
      elseif nfrq == 2
        fprintf(fil1MC,'Res    S2   dS2     Tf_ps   dTf_ps     Chi2       dChi2        R    dR      Tc      numMC    J(0)       dJ(0)        Phi     dPhi     Rex1  Rex2  dRex1 dRex2\n');
      elseif nfrq == 3
        fprintf(fil1MC,'Res    S2   dS2     Tf_ps   dTf_ps     Chi2       dChi2        R    dR      Tc      numMC    J(0)       dJ(0)        Phi     dPhi     Rex1  Rex2  Rex3  dRex1 dRex2 dRex3\n');
      elseif nfrq == 4
        fprintf(fil1MC,'Res    S2   dS2     Tf_ps   dTf_ps     Chi2       dChi2        R    dR      Tc      numMC    J(0)       dJ(0)        Phi     dPhi     Rex1  Rex2  Rex3  Rex4  dRex1 dRex2 dRex3 dRex4\n');
      elseif nfrq == 5
        fprintf(fil1MC,'Res    S2   dS2     Tf_ps   dTf_ps     Chi2       dChi2        R    dR      Tc      numMC    J(0)       dJ(0)        Phi     dPhi     Rex1  Rex2  Rex3  Rex4  Rex5 dRex1 dRex2 dRex3 dRex4 dRex5\n');
      elseif nfrq == 6
        fprintf(fil1MC,'Res    S2   dS2     Tf_ps   dTf_ps     Chi2       dChi2        R    dR      Tc      numMC    J(0)       dJ(0)        Phi     dPhi     Rex1  Rex2  Rex3  Rex4  Rex5  Rex6 dRex1 dRex2 dRex3 dRex4 dRex5 dRex6\n');
      end
    end

    eval(['fil2MC = fopen(''ls2_MC.' num2str(gnout) ''',''w'');']);		% Ouvre 'ls2_MC.out'
    eval(['fil3MC = fopen(''lse3_MC.' num2str(gnout) ''',''w'');']);		% Ouvre 'lse3_MC.out'
    fprintf(fil2MC,'Res    S2   dS2     Tf_ps   dTf_ps     Chi2       dChi2        R    dR      Tc      numMC\n');
    fprintf(fil3MC,'Res    S2s dS2s    S2f dS2f  S2s*f dS2s*f   Ts_ps   dTs_ps     Chi2       dChi2        R    dR      Tc      numMC\n');

    if nfrq > 1
      eval(['fil4MC = fopen(''lse4_MC.' num2str(gnout) ''',''w'');']);		% Ouvre 'lse4_MC.out'
      fprintf(fil4MC,'Res    S2s dS2s    S2f dS2f  S2s*f dS2s*f   Ts_ps   dTs_ps     Tf_ps   dTf_ps     Chi2       dChi2        R    dR      Tc      numMC\n');
    end


    for k = 1:nres                      % Boucle pour chaque residu

      fprintf('Residu: %g \n',seq(k,1));

      for i = 1:numMC                   % Boucle pour le Monte-Carlo pour 
                                        % le calcul d'erreurs des S2 et Tau

% Perturbation avec une matrice aleatoire avec une distribution
% normale de moyenne 0 et de variance 1.

		if fmy == 'y'				% Si "Fast-Mapping"
		  aleat = randn(nres,(nfrq+2));		% Matrice aleatoire
		else					% sinon
          aleat = randn(nres,(2*nfrq+1));		% Matrice aleatoire
		end

% SI LE PREMIER TOUR N'APPORTE PAS DE PERTURBATIONS, ALORS LE 'dY' EST NUL
% ET CELA POSE DES PROBLEMES POUR LE CALCUL DU 'Chi2'

        dY = dJomega(k,:) .* aleat(k,:);

        option=optimset('fminsearch');
%        option=optimset(option,'MaxIter',40000,'MaxFunEvals',50000);


        if rexy == 'y'                  % Si calcul du Rex pour une frequence

          s2MC = initprm_LSREX(k,1);
          taupMC = initprm_LSREX(k,2);
          phiMC = initprm_LSREX(k,3); 
          Y = calcYlsrex(k,:) + dY;     % Initialisation du fit avec le 'lambda.Rex'
          params = [s2MC taupMC phiMC];
          fitLSREX = fminsearch('LSREX',params,option,X,Y,dY,tauc,lwx2);	% Fit avec LSREX.m
          temp_LSREX(i,1) = fitLSREX(1);					% S2
          temp_LSREX(i,2) = (tauc*fitLSREX(2))/(tauc-fitLSREX(2))*1e12;		% Conversion taup > tauf
          calcYlsrexMC(i,:) = (2/5)*((fitLSREX(1)*tauc./(1+(X*tauc).^2))+((1-fitLSREX(1))*fitLSREX(2)./(1+(X*fitLSREX(2)).^2)))+(lwx2*abs(fitLSREX(3))*(exp(-(X).^6)));
          calcYlsrexMC_Rex(i,:) = (2/5)*((fitLSREX(1)*tauc./(1+(X*tauc).^2))+((1-fitLSREX(1))*fitLSREX(2)./(1+(X*fitLSREX(2)).^2)));    % ATTENTION, J(0) = J(0)obs - lambda.Rex
          error2 = (Y - calcYlsrexMC(i,:)).^2;
          temp_LSREX(i,3) = sum(error2./(calcYlsrexMC(i,:).^2));	% Calcul du Chi2
          R2 = corrcoef(Y,calcYlsrexMC(i,:));
          temp_LSREX(i,4) = R2(1,2);					% Calcul du R
          temp_LSREX(i,5) = abs(fitLSREX(3));				% Phi positif !!!!
        end

                                        % Initialisation du fit LS2
        s2MC = initprm_LS2(k,1);
        taupMC = initprm_LS2(k,2);
        Y = calcYls2(k,:) + dY;
        params = [s2MC taupMC];
        fitLS2 = fminsearch('LS2',params,option,X,Y,dY,tauc);		% Fit avec LS2.m
        temp_LS2(i,1) = fitLS2(1);					% S2
        temp_LS2(i,2) = (tauc*fitLS2(2))/(tauc-fitLS2(2))*1e12;		% Conversion taup > tauf
        calcYls2MC(i,:) = (2/5)*((fitLS2(1)*tauc./(1+(X*tauc).^2))+((1-fitLS2(1))*fitLS2(2)./(1+(X*fitLS2(2)).^2)));
        error2 = (Y - calcYls2MC(i,:)).^2;
        temp_LS2(i,3) = sum(error2./(calcYls2MC(i,:).^2));		% Calcul du Chi2
        R2 = corrcoef(Y,calcYls2MC(i,:));
        temp_LS2(i,4) = R2(1,2);					% Calcul du R

 
                                        % Initialisation du fit LSE3
        s2sMC = initprm_LSE3(k,1);
        s2fMC = initprm_LSE3(k,2);
        tauspMC = initprm_LSE3(k,3);
        Y = calcYlse3(k,:) + dY;
        params = [s2sMC s2fMC tauspMC];
        fitLSE3 = fminsearch('LSE3',params,option,X,Y,dY,tauc);		% Fit avec LSE3.m
        temp_LSE3(i,1) = fitLSE3(1);					% S2s
        temp_LSE3(i,2) = fitLSE3(2);					% S2f
        temp_LSE3(i,3) = (tauc*fitLSE3(3))/(tauc-fitLSE3(3))*1e12;	% Conversion tausp > taus
        calcYlse3MC(i,:) = (2/5)*((fitLSE3(1)*fitLSE3(2)*tauc./(1+(X*tauc).^2))+(fitLSE3(2)*(1-fitLSE3(1))*fitLSE3(3)./(1+(X*fitLSE3(3)).^2)));
        error2 = (Y - calcYlse3MC(i,:)).^2;
        temp_LSE3(i,4) = sum(error2./(calcYlse3MC(i,:).^2));		% Calcul du Chi2
        R2 = corrcoef(Y,calcYlse3MC(i,:));
        temp_LSE3(i,5) = R2(1,2);					% Calcul du R


        if nfrq > 1                     % Initialisation du fit LSE4
          s2sMC = initprm_LSE4(k,1);
          s2fMC = initprm_LSE4(k,2);
          tauspMC = initprm_LSE4(k,3);
          taufpMC = initprm_LSE4(k,4);
          Y = calcYlse4(k,:) + dY;
          params = [s2sMC s2fMC tauspMC taufpMC];
          fitLSE4 = fminsearch('LSE4',params,option,X,Y,dY,tauc);	% Fit avec LSE4.m
          temp_LSE4(i,1) = fitLSE4(1);					% S2s
          temp_LSE4(i,2) = fitLSE4(2);					% S2f
          temp_LSE4(i,3) = (tauc*fitLSE4(3))/(tauc-fitLSE4(3))*1e12;	% Conversion tausp > taus
          temp_LSE4(i,4) = (tauc*fitLSE4(4))/(tauc-fitLSE4(4))*1e12;	% Conversion taufp > tauf
          calcYlse4MC(i,:) = (2/5)*((fitLSE4(1)*fitLSE4(2)*tauc./(1+(X*tauc).^2))+(fitLSE4(2)*(1-fitLSE4(1))*fitLSE4(3)./(1+(X*fitLSE4(3)).^2))+((1-fitLSE4(2))*fitLSE4(4)./(1+(X*fitLSE4(4)).^2)));
          error2 = (Y - calcYlse4MC(i,:)).^2;
          temp_LSE4(i,5) = sum(error2./(calcYlse4MC(i,:).^2));		% Calcul du Chi2
          R2 = corrcoef(Y,calcYlse4MC(i,:));
          temp_LSE4(i,6) = R2(1,2);					% Calcul du R
        end

      end

      if rexy == 'y'				% Si calcul du Rex
        params_LSREXMC(k,1) = seq(k,1);
        params_LSREXMC(k,2) = mean(temp_LSREX(:,1));		% S2
        params_LSREXMC(k,3) = std(temp_LSREX(:,1));
        params_LSREXMC(k,4) = mean(temp_LSREX(:,2));		% Tf_ps
        params_LSREXMC(k,5) = std(temp_LSREX(:,2));
        params_LSREXMC(k,6) = mean(temp_LSREX(:,3));		% Chi2
        params_LSREXMC(k,7) = std(temp_LSREX(:,3));
        params_LSREXMC(k,8) = mean(temp_LSREX(:,4));		% R
        params_LSREXMC(k,9) = std(temp_LSREX(:,4));
        params_LSREXMC(k,10) = tauc;				% Tc
        params_LSREXMC(k,11) = numMC;				% numMC
        params_LSREXMC(k,12) = mean(calcYlsrexMC_Rex(:,1));	% J(0) = J(0)obs - lambda.Rex 
        params_LSREXMC(k,13) = std(calcYlsrexMC_Rex(:,1));
        params_LSREXMC(k,14) = mean(temp_LSREX(:,5));		% Phi
        params_LSREXMC(k,15) = std(temp_LSREX(:,5));
        if nfrq == 1
          params_LSREXMC(k,16) = wx2n(1,:) * mean(temp_LSREX(:,5));	% Valeur du Rex = wx2 . phi
          params_LSREXMC(k,17) = wx2n(1,:) * std(temp_LSREX(:,5));
          fprintf(fil1MC,'%-3d  %5.2f %4.2f  %8.2f %8.2f  %7.5e %7.5e  %5.3f %5.3f  %5.2e %5d  %7.5e %7.5e  %5.2e %5.2e  %5.3f %5.3f\n',params_LSREXMC(k,:));
        elseif nfrq == 2
          params_LSREXMC(k,16:17) = wx2n(1,:) * mean(temp_LSREX(:,5));
          params_LSREXMC(k,18:19) = wx2n(1,:) * std(temp_LSREX(:,5));
          fprintf(fil1MC,'%-3d  %5.2f %4.2f  %8.2f %8.2f  %7.5e %7.5e  %5.3f %5.3f  %5.2e %5d  %7.5e %7.5e  %5.2e %5.2e  %5.3f %5.3f %5.3f %5.3f\n',params_LSREXMC(k,:));
        elseif nfrq == 3
          params_LSREXMC(k,16:18) = wx2n(1,:) * mean(temp_LSREX(:,5));
          params_LSREXMC(k,19:21) = wx2n(1,:) * std(temp_LSREX(:,5));
          fprintf(fil1MC,'%-3d  %5.2f %4.2f  %8.2f %8.2f  %7.5e %7.5e  %5.3f %5.3f  %5.2e %5d  %7.5e %7.5e  %5.2e %5.2e  %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f\n',params_LSREXMC(k,:));
        elseif nfrq == 4
          params_LSREXMC(k,16:19) = wx2n(1,:) * mean(temp_LSREX(:,5));
          params_LSREXMC(k,20:23) = wx2n(1,:) * std(temp_LSREX(:,5));
          fprintf(fil1MC,'%-3d  %5.2f %4.2f  %8.2f %8.2f  %7.5e %7.5e  %5.3f %5.3f  %5.2e %5d  %7.5e %7.5e  %5.2e %5.2e  %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f\n',params_LSREXMC(k,:));
        elseif nfrq == 5
          params_LSREXMC(k,16:20) = wx2n(1,:) * mean(temp_LSREX(:,5));
          params_LSREXMC(k,21:25) = wx2n(1,:) * std(temp_LSREX(:,5));
          fprintf(fil1MC,'%-3d  %5.2f %4.2f  %8.2f %8.2f  %7.5e %7.5e  %5.3f %5.3f  %5.2e %5d  %7.5e %7.5e  %5.2e %5.2e  %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f\n',params_LSREXMC(k,:));
        elseif nfrq == 6
          params_LSREXMC(k,16:21) = wx2n(1,:) * mean(temp_LSREX(:,5));
          params_LSREXMC(k,22:27) = wx2n(1,:) * std(temp_LSREX(:,5));
          fprintf(fil1MC,'%-3d  %5.2f %4.2f  %8.2f %8.2f  %7.5e %7.5e  %5.3f %5.3f  %5.2e %5d  %7.5e %7.5e  %5.2e %5.2e  %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f %5.3f\n',params_LSREXMC(k,:));
        end
      end

      params_LS2MC(k,1) = seq(k,1);
      params_LS2MC(k,2) = mean(temp_LS2(:,1));
      params_LS2MC(k,3) = std(temp_LS2(:,1));
      params_LS2MC(k,4) = mean(temp_LS2(:,2));
      params_LS2MC(k,5) = std(temp_LS2(:,2));
      params_LS2MC(k,6) = mean(temp_LS2(:,3));
      params_LS2MC(k,7) = std(temp_LS2(:,3));
      params_LS2MC(k,8) = mean(temp_LS2(:,4));
      params_LS2MC(k,9) = std(temp_LS2(:,4));
      params_LS2MC(k,10) = tauc;
      params_LS2MC(k,11) = numMC;
      fprintf(fil2MC,'%-3d  %5.2f %4.2f  %8.2f %8.2f  %7.5e %7.5e  %5.3f %5.3f  %5.2e %5d\n',params_LS2MC(k,:));

      params_LSE3MC(k,1) = seq(k,1);
      params_LSE3MC(k,2) = mean(temp_LSE3(:,1));
      params_LSE3MC(k,3) = std(temp_LSE3(:,1));
      params_LSE3MC(k,4) = mean(temp_LSE3(:,2));
      params_LSE3MC(k,5) = std(temp_LSE3(:,2));
      params_LSE3MC(k,6) = params_LSE3MC(k,2) * params_LSE3MC(k,4);
      params_LSE3MC(k,7) = params_LSE3MC(k,6) * (params_LSE3MC(k,3)/params_LSE3MC(k,2)+params_LSE3MC(k,5)/params_LSE3MC(k,4));
      params_LSE3MC(k,8) = mean(temp_LSE3(:,3));
      params_LSE3MC(k,9) = std(temp_LSE3(:,3));
      params_LSE3MC(k,10) = mean(temp_LSE3(:,4));
      params_LSE3MC(k,11) = std(temp_LSE3(:,4));
      params_LSE3MC(k,12) = mean(temp_LSE3(:,5));
      params_LSE3MC(k,13) = std(temp_LSE3(:,5));
      params_LSE3MC(k,14) = tauc;
      params_LSE3MC(k,15) = numMC;
      fprintf(fil3MC,'%-3d  %5.2f %4.2f  %5.2f %4.2f  %5.2f %4.2f  %8.2f %8.2f  %7.5e %7.5e  %5.3f %5.3f  %5.2e %5d\n',params_LSE3MC(k,:));

      if nfrq > 1
        params_LSE4MC(k,1) = seq(k,1);
        params_LSE4MC(k,2) = mean(temp_LSE4(:,1));
        params_LSE4MC(k,3) = std(temp_LSE4(:,1));
        params_LSE4MC(k,4) = mean(temp_LSE4(:,2));
        params_LSE4MC(k,5) = std(temp_LSE4(:,2));
        params_LSE4MC(k,6) = params_LSE4MC(k,2) * params_LSE4MC(k,4);
        params_LSE4MC(k,7) = params_LSE4MC(k,6) * (params_LSE4MC(k,3)/params_LSE4MC(k,2)+params_LSE4MC(k,5)/params_LSE4MC(k,4));
        params_LSE4MC(k,8) = mean(temp_LSE4(:,3));
        params_LSE4MC(k,9) = std(temp_LSE4(:,3));
        params_LSE4MC(k,10) = mean(temp_LSE4(:,4));
        params_LSE4MC(k,11) = std(temp_LSE4(:,4));
        params_LSE4MC(k,12) = mean(temp_LSE4(:,5));
        params_LSE4MC(k,13) = std(temp_LSE4(:,5));
        params_LSE4MC(k,14) = mean(temp_LSE4(:,6));
        params_LSE4MC(k,15) = std(temp_LSE4(:,6));
        params_LSE4MC(k,16) = tauc;
        params_LSE4MC(k,17) = numMC;
        fprintf(fil4MC,'%-3d  %5.2f %4.2f  %5.2f %4.2f  %5.2f %4.2f  %8.2f %8.2f  %8.2f %8.2f  %7.5e %7.5e  %5.3f %5.3f  %5.2e %5d\n',params_LSE4MC(k,:));
      end

      if rexy == 'y'					% Si calcul du Rex
        Jlsrextemp(k,:) = mean(calcYlsrexMC_Rex);	% ATTENTION, J(0) = J(0)obs - lambda.Rex
        dJlsrextemp(k,:) = std(calcYlsrexMC_Rex);
      end

      Jls2temp(k,:) = mean(calcYls2MC);
      dJls2temp(k,:) = std(calcYls2MC);
      Jlse3temp(k,:) = mean(calcYlse3MC);
      dJlse3temp(k,:) = std(calcYlse3MC);

      if nfrq > 1
        Jlse4temp(k,:) = mean(calcYlse4MC);
        dJlse4temp(k,:) = std(calcYlse4MC);
      end

      %% RECUPERATION DES CHI2 DE CHAQUE RESIDU POUR L'ANALYSE STATISTIQUE...
	  if rexy == 'y'
	    chi2_LSREX_MC(:,k) = temp_LSREX(:,3);
	  end
	  chi2_LS2_MC(:,k) = temp_LS2(:,3);
	  chi2_LSE3_MC(:,k) = temp_LSE3(:,4);
	  if nfrq > 1
	    chi2_LSE4_MC(:,k) = temp_LSE4(:,5);
	  end

    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    seql = rot90(seq,1);		% Creation d'une sequence lineaire pour la matrice des Chi2
	
    if rexy == 'y'			% Si calcul du Rex...
      fclose(fil1MC);			% Ferme le fichier "lsrex_MC.out"

      chi2mat_LSREX = [ ]; chi2mat_LSREX = zeros(numMC+1,nres);	% Creation d'une matrice pour les Chi2
	  chi2mat_LSREX(1,:) = seql;
	  chi2mat_LSREX(2:numMC+1,:) = chi2_LSREX_MC;

      eval(['save Jtheo_lsrex_MC.' num2str(gnout) ' Jlsrextemp -ascii;']);
      eval(['save dJtheo_lsrex_MC.' num2str(gnout) ' dJlsrextemp -ascii;']);
      eval(['save CHI2_LSREX_MC.' num2str(gnout) ' chi2mat_LSREX -ascii;']);
      fprintf(' \n')
      fprintf(' Files "lsrex_MC.%s", "Jtheo_lsrex_MC.%s", "dJtheo_lsrex_MC.%s" and "CHI2_LSREX_MC.%s" are created. \n',gnout,gnout,gnout,gnout);

    end

    fclose(fil2MC);			% Ferme le fichier "ls2_MC.out"
	
    chi2mat_LS2 = [ ]; chi2mat_LS2 = zeros(numMC+1,nres);		% Creation d'une matrice pour les Chi2
	chi2mat_LS2(1,:) = seql;
	chi2mat_LS2(2:numMC+1,:) = chi2_LS2_MC;
	
    fclose(fil3MC);			% Ferme le fichier "lse3_MC.out"

    chi2mat_LSE3 = [ ]; chi2mat_LSE3 = zeros(numMC+1,nres);		% Creation d'une matrice pour les Chi2
	chi2mat_LSE3(1,:) = seql;
	chi2mat_LSE3(2:numMC+1,:) = chi2_LSE3_MC;

    if nfrq > 1
      fclose(fil4MC);			% Ferme le fichier "lse4_MC.out"
	  
	  chi2mat_LSE4 = [ ]; chi2mat_LSE4 = zeros(numMC+1,nres);	% Creation d'une matrice pour les Chi2
	  chi2mat_LSE4(1,:) = seql;
	  chi2mat_LSE4(2:numMC+1,:) = chi2_LSE4_MC;
    end

    eval(['save Jtheo_ls2_MC.' num2str(gnout) ' Jls2temp -ascii;']);
    eval(['save Jtheo_lse3_MC.' num2str(gnout) ' Jlse3temp -ascii;']);
    eval(['save dJtheo_ls2_MC.' num2str(gnout) ' dJls2temp -ascii;']);
    eval(['save dJtheo_lse3_MC.' num2str(gnout) ' dJlse3temp -ascii;']);
    eval(['save CHI2_LS2_MC.' num2str(gnout) ' chi2mat_LS2 -ascii;']);
    eval(['save CHI2_LSE3_MC.' num2str(gnout) ' chi2mat_LSE3 -ascii;']);
    fprintf(' \n')
    fprintf(' Files "ls2_MC.%s", "Jtheo_ls2_MC.%s", "dJtheo_ls2_MC.%s" and "CHI2_LS2_MC.%s" are created. \n',gnout,gnout,gnout,gnout);
    fprintf(' \n')
    fprintf(' Files "lse3_MC.%s", "Jtheo_lse3_MC.%s", "dJtheo_lse3_MC.%s" and "CHI2_LSE3_MC.%s" are created. \n',gnout,gnout,gnout,gnout);

    if nfrq > 1
      eval(['save Jtheo_lse4_MC.' num2str(gnout) ' Jlse4temp -ascii;']);
      eval(['save dJtheo_lse4_MC.' num2str(gnout) ' dJlse4temp -ascii;']);
      eval(['save CHI2_LSE4_MC.' num2str(gnout) ' chi2mat_LSE4 -ascii;']);
      fprintf(' \n')
      fprintf(' Files "lse4_MC.%s", "Jtheo_lse4_MC.%s", "dJtheo_lse4_MC.%s" and "CHI2_LSE4_MC.%s" are created. \n',gnout,gnout,gnout,gnout);
    end

  end

end

fprintf('\n\n')
fprintf('The program is finished.\n')
fprintf('Now, you must analyse your results.\n')
fprintf('Good luck !!\n')

